<?php

namespace Drupal\qrcode_fields\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Plugin implementation 'qrcode_url'.
 *
 * @FieldType(
 *   id = "qrcode_url",
 *   label = @Translation("URL"),
 *   description = @Translation("Field for generating QR codes from content entity."),
 *   category = @Translation("QR Code"),
 *   default_widget = "qrcode_url_field_widget",
 *   default_formatter = "qrcode_fields_formatter"
 * )
 */
class QRFieldUrl extends FieldItemBase {

	/**
	 * {@inheritdoc}
	 */
	public static function defaultFieldSettings() {
		return [
			'qrcode_plugin' => 'gchart',
		] + parent::defaultFieldSettings();
	}

	/**
	 * {@inheritdoc}
	 */
	public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
		$elements = [];
		$pluginDefinitions = \Drupal::service('plugin.manager.qrcode_fields')->getDefinitionsList();
		$elements['qrcode_plugin'] = [
			'#title' => $this->t('QR code url service plugin'),
			'#type' => 'select',
			'#options' => $pluginDefinitions,
			'#default_value' => $this->getSetting('qrcode_plugin'),
			'#description' => $this->t('Service to use for QR code generation.'),
		];
		return $elements;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
		$properties['url'] = DataDefinition::create('string')
			->setLabel(t('QR code url'));
		return $properties;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function schema(FieldStorageDefinitionInterface $field_definition) {
		return [
			'columns' => [
				'url' => [
					'type' => 'varchar',
					'length' => '255',
				],
			],
		];
	}

}
